import os, os.path
import shutil
import sys
import re
from collections import deque
import random
import math

import pdb

def parseListFile(fn, labelre):
    """ Eg, Topic 0 followed by linebreaked list of words, etc """
    vals = []
    for line in open(fn):
        if(labelre.match(line)):
            vals.append([])
        elif(len(line.strip()) > 0):            
            vals[-1].append(line.strip())
    return vals

def kfold(vals, k):
    """ Split into k train-test folds"""
    random.shuffle(vals)
    foldsize = float(len(vals)) / k

    (test, train) = ([], [])
    for ki in range(k):
        start = int(math.floor(foldsize * ki))
        end = int(math.floor(foldsize * (ki + 1)))
        r = deque(vals)
        r.rotate(-1 * start)
        test.append(list(r)[:(end-start)])
        train.append(list(r)[(end-start):])

    return (test, train)

# Get all positive words for each concept
seeds = parseListFile('seedwords.txt', re.compile('Seed \d+'))
pos = parseListFile('poswords.txt', re.compile('Topic \d+'))
allpos = [list(set(s).union(set(p))) for (s, p) in zip(seeds, pos)]

# Read in the vocabulary
vocab = [line.strip() for line in open('stewart.vocab')]

# Split into k=4 train/test folds for each concept
k = 4
os.mkdir('folds')

for (ci, cwords) in enumerate(allpos):
    # For each concept

    (test, train) = kfold(cwords, k)
    for (fi, (testf, trainf)) in enumerate(zip(test, train)):
        # For each fold
        
        outf = open(os.path.join('folds','c%d-f%d.test' % (ci, fi)), 'w')
        for testword in testf:
            outf.write('%s\n' % testword)
        outf.close()

        outf = open(os.path.join('folds','c%d-f%d.train' % (ci, fi)), 'w')
        for trainword in trainf:
            outf.write('%s\n' % trainword)
        outf.close()    
