"""
Convert Python Dataset objects to directories for suitable for
consumption by Java LogicLDA code
"""
import os, os.path
import cPickle as CP

import numpy as NP

from LogicExperiments import Dataset

# Dataset info comes from NEW-GRID/GridRun.py
#
dsdir = '/ua/andrzeje/private/logicLDA/code/NEW-GRID/datasets'
datasets = [('20news-allcomp', 20),
            ('congress', 25),
            ('macvspc', 20),
            ('polarity', 20),
            ('s1MLAB', 2),
            ('s2CLAB', 3),
            ('stewart100-hdg-sent', 100)]

def matWrite(mat, fn):
    outf = open(fn, 'w')
    for ri in range(mat.shape[0]):
        for ci in range(mat.shape[1]):
            outf.write('%f ' % mat[ri,ci])
        outf.write('\n')
    outf.close()

for (dataset, T) in datasets:
    print 'Populating %s' % dataset
    # Load Python LogicExperiments.Dataset instance
    oldds = CP.load(open(os.path.join(dsdir, dataset)))
    ds = Dataset(oldds.flatw,oldds.flatd,
                 oldds.vocab,oldds.docs,
                 oldds.name,oldds.filename,oldds.notes,
                 oldds.metadata)
    W = len(ds.vocab)
    
    # Dump out to Java-friendly plaintext files
    ds.exportToJavaDir(os.path.join(os.getcwd(), dataset), dataset)

    # Output *.alpha and *.beta parameters

    # ICML synthetics require custom alpha/beta
    if(dataset == 's1MLAB' or dataset == 's2CLAB'):
        (alpha, beta) = (float(1), 0.01)
    else:
        (alpha, beta) = (float(50) / T, 0.01)

    alphaFilename = os.path.join(
        os.path.join(os.getcwd(), dataset), '%s.alpha' % dataset)
    matWrite(alpha * NP.ones((1,T)), alphaFilename)

    betaFilename = os.path.join(
        os.path.join(os.getcwd(), dataset), '%s.beta' % dataset)
    matWrite(beta * NP.ones((T,W)), betaFilename)    

    # randseed = 194582
    # (numsamp, numiter, freq) = (2000,100,1000)
    # p.numouter = 100
    # p.numinner = 100000
    # (p.stepa, p.stepb) = (NP.sqrt(p.numinner), p.numinner)
