"""
Run (parallel) standard LDA, write out the final sample along with
estimated phi/theta
"""

import pdb

import numpy as NP
import numpy.random as NPR

import sys
from pSSLDA import infer
import FastLDA 

def readInt(filename):
    # First pass
    N = 0
    for line in open(filename):
        N += len(line.split())
    # Populate NumPy array
    retval = NP.zeros((N,), dtype=NP.int)
    print str(retval)
    i = 0
    for line in open(filename):
        for val in line.split():
            retval[i] = int(val)
            i += 1
    return retval

def readMatrix(filename):
    # First pass
    (rows, cols) = (0, 0)
    for line in open(filename):
        if(cols == 0):
            cols = len(line.split())
            rows += 1
        elif(len(line.split()) == cols):
            rows += 1
    # Populate NumPy array
    retval = NP.zeros((rows,cols))
    (row, col) = (0, 0)
    for (ri, line) in enumerate(open(filename)):
        for (ci, val) in enumerate(line.split()):
            if(ci < cols and ri < rows):
                retval[ri,ci] = float(val)
    return retval


    
# Example usage (2000 samples using 16 cores)
#
# python ./scripts/fastStandard.py ./stewart/stewart 2000 16 194582
#
basefn = sys.argv[1]
numsamp = int(sys.argv[2])
P = int(sys.argv[3])
randseed = int(sys.argv[4])

# Load input data
#
docs = readInt('%s.docs' % basefn)
words = readInt('%s.words' % basefn)
alpha = readMatrix('%s.alpha' % basefn)
beta = readMatrix('%s.beta' % basefn)

# Load z-labels
#
zlabels = []
for line in open('%s.zlabel' % basefn):
    vals = [float(tok) for tok in line.split()]
    if(all([val == 0 for val in vals])):
        zlabels.append(None)
    else:
        zlabels.append(NP.array(vals, dtype = NP.float))

# Dimensionalities
D = docs.max() + 1
T = beta.shape[0]
W = beta.shape[1]
N = words.shape[0]

# Do parallel inference
finalz = infer(words, docs, alpha, beta, numsamp, randseed, P,
               zlabels = zlabels)

# Dump sample out to disk
outf = open('%s.zl-init' % basefn, 'w')
for (i, zi) in enumerate(finalz):
    if((i+1) % 1000 == 0):
        outf.write('\n')
    outf.write('%d ' % zi)
outf.close()
        
